# -*- coding: utf-8 -*-
#
# Author: Chris Oliver (excid3@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import os
import subprocess

from . import config


class FreeSpaceUnavailable(Exception):
    """This means there's no way to get the free space on some device."""
    pass

try:
    from win32file import GetDiskFreeSpaceEx
except ImportError:
    def GetDiskFreeSpaceEx(drive):
        """This is a placeholder for if win32file is unavailable.
        
        :param str drive: the name of the drive to check for free space
        """
        raise FreeSpaceUnavailable


def drive_free_space():
    """Retrieve the current working directory's free space.
    
    :return: the remaining space in KiB, MiB, or GiB
    :rtype: str
    """
    if os.name == 'posix':  # get free space if they have statvfs
        data = os.statvfs(config.cwd)
        return convert_file_size(float(data[0] * data[4]))
    if os.name == 'nt':
        drive = os.path.splitdrive(config.cwd)[0]  # Gets drive letter
        try:
            d_size = GetDiskFreeSpaceEx(drive)
        except FreeSpaceUnavailable:
            return 'Unavailable'
        return convert_file_size(d_size[2])
    # unsupported OS, return error
    return 'Unavailable'


def convert_file_size(num_bytes):
    """Convert file size from bytes to appropriate size.
    
    :param int num_bytes: the number of bytes to be formatted
    :return: a nice representation of the size, in KiB, MiB, or GiB
    :rtype: str
    """
    kb = num_bytes / 1024.0
    if kb < 1000:
        return '%.1f %s' % (kb, 'KiB')
    mb = kb / 1024.0
    if mb < 1000:
        return '%.1f %s' % (mb, 'MiB')
    gb = mb / 1024.0
    return '%.2f %s' % (gb, 'GiB')


def utf(desc):
    """Convert the string to a fully UTF compatible format."""
    new_word = ''
    for letter in desc:
        if ord(letter) > 128:
            letter = chr(127)
        new_word += letter
    return new_word


def join_url(first, *args):
    """Combines the given elements into a full URL.
    
    :param str first: the first part of the URL
    :param str args: subsequent parts of the URL
    :return: a complete URL from the given parts
    :rtype: str
    """
    if len(args) == 0:
        return first
    elif len(args) == 1:
        return first.strip('/') + '/' + args[0].strip('/')
    else:
        return join_url(first.strip('/') + '/' + args[0].strip('/'), *args[1:])


def run_root(program, args=''):
    """Run the given program with the given arguments, gaining root privileges
    using some GUI-based su/sudo program. The first available from among the 
    choices gksu, gksudo, kdesu, and kdesudo will be used.

    :param str program: the command to be run
    :param str args: the arguments to run with the command
    :return: a success/error code and a message
    :rtype: list(int, str)
    """
    # Find an available privilege-escalating tool.
    root = ''
    for a in ('gksu', 'gksudo', 'kdesu', 'kdesudo'):
        if subprocess.run('which ' + a).returncode == 0:
            root = a
            break

    # Give compatible GUI su's a shorter command description
    if root.startswith('gk'):
        description = '%s %s' % (program, args)
        if len(description) > 50:
            description = '--description \'%s...\'' % (description[:46])
    else:  # Certain su GUIs do not accept alternate descriptions
        description = ''

    # If no tool was available, return an error code.
    # TODO: actually raise an error.
    if root == '':
        return 64, 'Could not find superuser access'

    # Attempt to run the program as root.
    proc = subprocess.run('%s %s -- %s %s' %
                          (root, description, program, args))
    return proc.returncode, ''
