# -*- coding: utf-8 -*-
#
# Author: Chris Oliver (excid3@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import os
import platform
import shutil
import subprocess
from gettext import gettext as _
from os.path import join as path_join

import lib
from lib import config
from lib import log
from lib import run_root
from lib.source import Source
from lib.package import Package
from lib.version import version_compare


FILE = 'debian.conf'

# Set this plugin's attributes
NAME = 'Debian'
TYPE = 'OS'
VERSION = '0.92.4'
AUTHOR = 'Chris Oliver <excid3@gmail.com>'


def is_os():
    """Determines whether the current OS uses APT
    
    :return: true if the current OS uses APT, false otherwise
    :rtype: bool
    """
    return os.path.exists('/etc/apt/')


def create_project(name):
    """Creates project files
    
    :param str name: the name of the new project being created
    """
    dir_new = path_join(config.projects_dir, name)

    # Copy relevant files to the project directory.
    __grab_files(dir_new)

    # Write project info to a file in the project directory.
    with open(path_join(dir_new, FILE), 'w') as outfile:
        outfile.write('Computer Name: %s\n' % get_comp_name() +
                      'OS Name: %s\n' % get_os_name() +
                      'OS Version: %s\n' % get_os_version() +
                      'Architecture: %s\n' % get_os_arch() +
                      'Kernel: %s\n' % get_kernel_name())


def load_local_package_list(directory, arch):
    """Load package information from locally-stored lists.
    
    :param str directory: the directory to load lists from
    :param str arch: the OS architecture to load lists for
    :return: a dict populated with package information
    :rtype: dict
    """
    list_dir = path_join(directory, 'lists')
    debs = __parse_sources(directory)
    filenames = __files_from_debs(debs, arch)
    installed = __get_installed(directory)

    # Initialize vars
    packages = {}

    for item in filenames:
        # TODO: better enforce zipping/unzipping of list files.
        path = path_join(list_dir, item)
        if not os.path.exists(path):
            path = path.strip('.gz')
        with open(path, 'r') as data:
            url = 'http://%s' % item.split('_dists')[0].replace('_', '/')
            # Append all packages to list
            packages = __read_packages(data, installed, packages, url)

    return packages


def load_remote_package_list(directory, arch):
    """Get information to load package lists from the project's repos.
    
    :param str directory: the project's directory, to load source info from
    :param str arch: '32bit' or '64bit'
    :return: pairs of (url, destination) pairs
    :rtype: list(list)
    """
    list_dir = path_join(directory, 'lists')
    debs = __parse_sources(directory)
    # TODO: extract hashes for list files from Release files
    # and 'return zip(urls, temp_names, checksums)' instead
    temp_names = __files_from_debs(debs, arch, list_dir)
    urls = __urls_from_debs(debs, arch)

    # FIXME: don't return a list, just fix everything that expects a list
    # instead of an iterator.
    return list(zip(urls, temp_names))  # Returns urls, gzip file locations


def get_dependencies(directory, all_packages, package_name):
    """Takes package name and returns package information for each dependency 
    (recursive).
    
    :param str directory: the directory to place downloaded packages in
    :param dict all_packages: the dict holding all the package info
    :param str package_name: the name of the package to get dependencies for
    :return: a list of URLs to download package dependencies
    :rtype: list(str)
    """
    urls = []
    if package_name not in all_packages:
        return {}
    values = all_packages.get(package_name)
    depends = values.depends.split(', ')
    depends += values.recommends.split(', ')  # APT treats recommends as depends
    depends += values.pre_depends.split(', ')  # pre-depends are important too
    filename = values.filename.split('/')  # Get the parts for the pkg filename
    urls.append((values.filename,
                 path_join(directory, filename[-1]),
                 values.checksums))
    values.installed_version = values.version
    all_packages[package_name] = values

    for item in depends:
        data = item.split()
        if not data == []:
            # Get dependencies for this package too if it isn't already
            # installed
            if data[0] in all_packages \
                    and not all_packages.get(data[0]).installed_version:
                urls += get_dependencies(directory,
                                         all_packages,
                                         data[0])

    return urls


def get_sources(directory):
    """Get the filename of main sources file, relative to directory.
    
    :param str directory: the directory holding APT state files
    :return: the path to the sources.list file
    :rtype: str
    """
    return path_join(directory, 'sources', 'sources.list')


def install_cache(project_dir, script_name, move=False):
    """Transfer index files to the APT cache. Uses sh script called as 'root'.
    
    :param str project_dir: the directory holding the project
    :param str script_name: the name of the script which will place the files
    :param bool move: flags whether files should be moved rather than copied
    :return: flag whether the process was successful
    :rtype: bool
    """
    packs_dir = path_join(project_dir, 'packages')
    # Make sure there are packages to install
    if not os.path.exists(packs_dir):
        log.error(_('%s does not exist: no packages to be installed') %
                  packs_dir)
        return False

    # If we're missing the partial dir, APT will throw a fit.
    partial_dir = path_join(packs_dir, 'partial')
    if not os.path.exists(partial_dir):
        os.mkdir(partial_dir)

    script_path = path_join(project_dir, '%s.sh' % script_name)
    shutil.copyfile(path_join(config.cwd, 'lib', 'install.sh'), script_path)

    if move:
        transfer = '-move'
    else:
        transfer = ''

    log.info(_('install_cache: launching script as root'))
    run = run_root('sh', '%s %s %s' % (script_path, project_dir, transfer))
    if run[0] != 0:
        log.error('exit code:%i\n%s' % (run[0], run[1]))
        log.info(_('install_cache: failed'))
        return False
    log.info(_('install_cache: transfer success'))
    return True


def install_packs(project_dir, pack_names):
    """Install the listed packages using downloaded package files in the given
    project directory.
    
    :param str project_dir: the directory of the current project
    :param list(str) pack_names: names of the packages to be installed
    :return: a flag whether the process was successful
    :rtype: bool
    """
    packs_dir = path_join(project_dir, 'packages')
    run = run_root('xterm',
                   '-e sh -c \"apt-get -y -o '
                   'dir::cache::archives=\\\"%s\\\" '
                   '--allow-unauthenticated install %s; echo \\\"'
                   'Press [ENTER] to exit.\\\"; '
                   'read x\"' % (packs_dir, pack_names))
    if run[0] != 0:
        log.error(_('exit code:%i\n%s' % (run[0], run[1])))
        log.info(_('Failed to start Xterm as root'))
        return False
    return True


def update_status(out_dir, status_path='/var/lib/dpkg/status'):
    """Update the project's status file (list of installed packages).
    
    :param str out_dir: the directory to place the status file in
    :param str status_path: the path to the new status file
    :return: a flag whether the process was successful
    :rtype: bool
    """
    lists_dir = path_join(out_dir, 'lists')
    outfile = path_join(lists_dir, 'status')
    outfile_bak = path_join(lists_dir, 'status.bak')
    # Back up the current status file.
    run_root('cp', '%s %s' % (outfile, outfile_bak))
    # Copy in the new status file.
    run_root('cp', '%s %s' % (status_path, outfile))
    # TODO: check for error from 'cp', return False
    return True  # Everything went well.


def __read_packages(infile, installed, packages, main_url=''):
    """Read package info from the given index file, include information from the
    list of currently-installed packages, and add it to the current list of
    packages.
    
    :param File infile: the opened index file to read package data from
    :param list installed: the list of packages installed on the system
    :param dict packages: the list of already-parsed packages
    :return: packages dictionary with additional package info added
    :rtype: dict
    """
    # Create a new package object. We don't know its name yet.
    current = Package('', checksums={})

    for line in infile:
        # Excuse how unreadable this next section is. It could be implemented by
        # checking .startswith('Keyname:') for each one, but that's slow. Don't
        # Try to narrow these down further unless you get a full listing of
        # index file key names. False positives abound.
        if line[0] == 'P':
            # Pre-depends
            if line[0:3] == 'Pre':
                current.pre_depends = line[13:-1]
            # Package
            elif line[0:8] == 'Package:':
                current.name = line[9:-1]
        # Version
        elif line[0] == 'V' and line[0:3] == 'Ver':
            current.version = line[9:-1]
        elif line[0] == 'D':
            # Depends
            if line[0:3] == 'Dep':
                current.depends = line[9:-1]
            # Description
            elif line[0:12] == 'Description:':
                current.description = lib.utf(line[13:-1])
        # Filename
        elif line[0] == 'F':
            current.filename = lib.join_url(main_url, line[10:-1])
        elif line[0] == 'S':
            # Size
            if line[1] == 'i':
                current.size = int(line[6:-1])
            # SHA1
            elif line[3] == '1':
                current.checksums['SHA1'] = line[6:-1]
            # SHA256
            elif line[3] == '2':
                current.checksums['SHA256'] = line[8:-1]
        # Recommends
        elif line[0] == 'R' and line[0:3] == 'Rec':
            current.recommends = line[12:-1]
        # MD5sum
        elif line[0] == 'M' and line[1] == 'D':
            current.checksums['MD5sum'] = line[8:-1]
        elif line[0] == '\n' and current.name != '':
            # Finished reading this package, append it
            # Set the packages installed version
            __update_package(current, installed, packages)
            packages[current.name] = current
            current = Package('', checksums={})

    return packages


def __update_package(package, installed, packages):
    """Update package information based on packages already installed.
    
    :param Package package: the package to be compared to installed packages
    :param dict installed: the dict of currently installed packages
    :param dict packages: the dict of already-parsed packages
    """
    # Package already exists so update values if need be
    if package.name in packages:
        if version_compare(packages[package.name].version, package.version) \
                == 2:
            return  # If it's an older one, skip it

    # If there's no installed version, move on.
    if package.name not in installed:
        return

    installed_version = installed[package.name].version
    package.status = version_compare(installed_version, package.version)
    package.installed_version = installed_version

    packages[package.name] = package  # Set package info


def __grab_files(directory):
    """Grab files necessary for a Debian project.
    
    :param str directory: the directory to place APT files in
    """
    dir_lists = path_join(directory, 'lists')
    dir_sources = path_join(directory, 'sources')

    shutil.copytree('/etc/apt/', dir_sources)
    try:
        shutil.copytree('/var/lib/apt/lists/', dir_lists)
    except shutil.Error:
        # Certain files in this directory require root. There's no more specific
        # error to catch. If for some reason not all the necessary files get
        # copied, that'll be dealt with in the code that uses them.
        pass
    shutil.copyfile('/var/lib/dpkg/status',
                    path_join(dir_lists, 'status'))
    shutil.copyfile('/var/lib/dpkg/status',
                    path_join(dir_lists, 'status.bak'))


def __parse_sources(directory):
    """Get a list of all deb entries for project.
    
    :param str directory: the directory holding source files to be parsed
    :return: a list of deb source lines from source files in the given dir
    :rtype: list
    """
    sources = []
    sources_dir = path_join(directory, 'sources')

    # Parse main source file
    main_source = path_join(sources_dir, 'sources.list')
    if os.path.exists(main_source):
        sources += __parse_source_file(main_source)

    # Parse extra source files
    sources_d_dir = path_join(sources_dir, 'sources.list.d')
    if os.path.exists(sources_d_dir):
        for item in os.listdir(sources_d_dir):
            if item.endswith('.list'):
                sources += __parse_source_file(path_join(sources_d_dir, item))

    return sources


def __parse_source_file(location):
    """Get a specific source file's deb (repo/source) entries.
    
    :param str location: the path to the source file to be parsed
    :return: a list of deb lines from this file
    :rtype: list
    """
    found = []
    infile = open(location)
    for line in infile:
        # TODO: Add support for more protocols
        if line.startswith('deb http://'):
            if line.endswith('\n'):
                found.append(line[:-1])
            else:
                found.append(line)
    return found


def __files_from_debs(deb_list, arch, directory=''):
    """Generates package index filenames based on the given entries from source 
    file "deb" entries.
    
    :param list deb_list: a list of deb source lines 
    :param str arch: either '32bit' or '64bit'
    :param str directory: the directory to prepend to the retrieved filenames
    :return: the file names/paths for the index files related to the given repos
    :rtype: generator
    """
    for line in deb_list:
        source = Source(line, arch)
        for filename in source.filenames:
            path = path_join(directory, filename)
            yield path


def __urls_from_debs(deb_list, arch):
    """Return URLs for all index files associated with the repos specified in
    deb_list.
    
    :param list deb_list: a list of deb source lines
    :param str arch: either '32bit' or '64bit'
    :return: the URLs for the index files related to the given repos
    :rtype: generator
    """
    for line in deb_list:
        source = Source(line, arch)
        for url in source.urls:
            yield url


def __get_installed(directory):
    """Get a list of installed packages.
    
    :param str directory: the directory holding package info files
    :return: a dict of installed Packages with package.name as their keys
    :rtype: dict(Package)
    """
    with open(path_join(directory, 'lists', 'status'), 'r') as status:
        installed = {}
        current = Package('')
        for line in status:
            if line.startswith('Package:'):
                current.name = line[9:-1]
            elif line.startswith('Version:'):
                current.version = line[9:-1]
            elif line.startswith('Status:'):
                current.status = line[8:-1]
            elif line.startswith('\n') \
                    and current.status == 'install ok installed':
                installed[current.name] = current
                current = Package('')

    return installed


def get_comp_name():
    """Get the name of this computer.
    
    :return: the name of this computer
    :rtype: str
    """
    system, node, release, version, machine, processor = platform.uname()
    return system


def get_os_name():
    """Get the name of this computers operating system.
    
    :return: the computer's OS name
    :rtype: str
    """
    process = subprocess.run(['cat', '/etc/issue.net'], stdout=subprocess.PIPE)
    output = str(process.stdout)
    if process.returncode != 0:
        log.error(_('Problem retrieving Debian version'))
    os_name = output.split()[0]
    return os_name


def get_os_version():
    """Get the version of this computer's operating system.
    
    :return: the OS version
    :rtype: str
    """
    process = subprocess.run(['cat', '/etc/issue.net'], stdout=subprocess.PIPE)
    output = str(process.stdout)
    if process.returncode != 0:
        log.error(_('Problem retrieving Debian version'))
    os_version = output.split()[1]
    return os_version


def get_os_arch():
    """Get the architecture of this computer's operating system. Return value is 
    either '32bit' or '64bit'.
    
    :return: the OS's architecture
    :rtype: str
    """
    # platform doesn't reliably get the architecture when compiled. So get
    # it more cleverly.
    process = subprocess.run(['apt-get', '-v'], stdout=subprocess.PIPE)
    apt_version = str(process.stdout)
    if 'i386' in apt_version:
        os_arch = '32bit'
    elif 'amd64' in apt_version:
        os_arch = '64bit'
    else:
        log.error(_('Unable to detect architecture. Defaulting to 64-bit. '
                    'If this is incorrect, edit your project\'s '
                    'debian.conf.'))
        os_arch = '64bit'
    return os_arch


def get_kernel_name():
    """Get the name of this computer's kernel.
    
    :return: the name of the kernel
    :rtype: str
    """
    system, node, release, version, machine, processor = platform.uname()
    return release
