import os.path
import urllib
import wx
import wx.lib.delayedresult as delayedresult
from gettext import gettext as _

from lib import log
from lib import config


class Thread(wx.Frame):
    """This demos simplistic use of delayedresult module."""
    def __init__(self, parent, func, endfunc):
        wx.Frame.__init__(self, None, title=_("Loading..."))
        self.parent = parent
        self.function = func
        self.endfunction = endfunc

        self.SetIcon(wx.Icon(config.ico_path, wx.BITMAP_TYPE_ICO))
        panel = wx.Panel(self)
        loading = wx.StaticText(panel, -1, _("This may take a while. Please be "
                                             "patient."))
        self.gauge = wx.Gauge(panel)

        main = wx.BoxSizer(wx.VERTICAL)
        main.Add(loading, 0, wx.ALIGN_CENTER_HORIZONTAL | wx.ALL, 5)
        main.Add(self.gauge, 0, wx.EXPAND | wx.ALL, 5)

        panel.SetSizer(main)
        self.SetSize(wx.Size(300, 100))
        self.Center()
        self.Show()

        self.jobID = 0
        self.abortEvent = delayedresult.AbortEvent()
        self.Bind(wx.EVT_CLOSE, self.on_close)
        self.Bind(wx.EVT_TIMER, self.timer_handler)

        self.timer = wx.Timer(self)
        self.timer.Start(100)
        
        self.parent.Enable(False)
        self.handle_get(None)

    def timer_handler(self, event):
        self.gauge.Pulse()
        
    def on_close(self, event):
        """Only needed because in demo, closing the window does not kill the 
        app, so worker thread continues and sends result to dead frame; normally
        your app would exit so this would not happen.
        """
        #if self.buttonAbort.IsEnabled():
            #self.log( "Exiting: Aborting job %s" % self.jobID )
            #self.handle_abort(None)
        self.Show()
            
    def handle_get(self, event):
        """Compute result in separate thread, doesn't affect GUI response."""
        self.abortEvent.clear()
        self.jobID += 1
        
        delayedresult.startWorker(self._result_consumer, self.function,
                                  wargs=(self.jobID, self.abortEvent),
                                  jobID=self.jobID)

    def _result_producer(self, job_id, abort_event):
        """Downloads the files in self.files"""
        
        msg = "Downloading %i file(s)\n" % len(self.files)
        wx.CallAfter(self.log_message, msg)

        success = True
        for data in self.files:
            if abort_event():
                return [1, self.files]

            url = data[0]
            file = data[1]
            end = url.split('/')
            protocol = end[0] + '//'
            site = end[2]
            end = end[len(end) - 1]

            msg = "%s%s    %s" % (protocol, site, end)
            wx.CallAfter(self.set_file, msg)

            try:  # Attempt to download the file
                urllib.urlretrieve(url, file, self.progress)
                # TODO: Generate md5hashes for these files
                
                msg = "Success: %s\nSaved to: %s\n" % (url, file)
                wx.CallAfter(self.log_message, msg)
    
            except IOError as e:  # Failed downloading
                success = False
                msg = "Failed: %s\nReason: %s\n" % (url, str(e))
                wx.CallAfter(self.log_message, msg)

        if success:
            wx.CallAfter(self.display_message, _("All downloads have been "
                                                 "completed successfully."),
                         _("Download Complete"))
            result = [0, self.files]
        else:
            wx.CallAfter(self.display_message, _("Some downloads failed to complete.") + "\n" + _("Please check") + " " + config.log_path + " " + _("for more details."), _("Download Failed"))
            result = [1, self.files]
        return result

    def progress(self, blocks, size, total):
        if blocks*size > total:
            fraction = float(total)/float(total)
        else:
            fraction = float(blocks*size)/float(total)
        
        wx.CallAfter(self.set_gauge, int(round(fraction * 100, 2)))

    def log_message(self, msg):
        log.info(msg)
        self.status.AppendText(msg)

    def display_message(self, msg, caption):
        wx.MessageBox(msg, caption)

    def set_gauge(self, val):
        self.download_gauge.SetValue(val)

    def set_file(self, val):
        self.current.SetLabel(val)

    def handle_abort(self, event):
        """Abort the result computation."""
        log.info("Aborting result for job %s" % self.jobID)
        #self.buttonGet.Enable(True)
        #self.buttonAbort.Enable(False)
        self.abortEvent.set()

    def _result_consumer(self, delayed_result):
        jobID = delayed_result.getJobID()
        assert jobID == self.jobID
        result = delayed_result.get()

        # output result
        #log.info( "Got result for job %s: %s" % (jobID, result) )
        self.parent.Enable()
        self.Destroy()
        if self.endfunction: self.endfunction()
