import os.path
import urllib.request
import wx
import hashlib
import wx.lib.delayedresult as delayedresult
from gettext import gettext as _

import lib
from lib import log
from lib import config


class Download(wx.Frame):
    """This demos simplistic use of delayedresult module."""
    def __init__(self, parent, endfunc, files, extract=False, overwrite=False):
        self.theme = config.theme
        wx.Frame.__init__(self, parent, title=_('Downloading...'))
        self.parent = parent
        self.files = files
        self.function = endfunc
        self.extract = extract
        self.overwrite = overwrite
        self.retries = 1

        # TODO: Add overwrite function

        self.SetIcon(wx.Icon(config.ico_path, wx.BITMAP_TYPE_ICO))
        panel = wx.Panel(self)
        loading = wx.StaticText(panel, -1, _('This may take a while. '
                                             'Please be patient.'))
        self.gauge = wx.Gauge(panel)

        self.cur = wx.StaticText(panel, -1, _('Current Transfer:'))
        self.cur.SetFont(wx.Font(8,
                                 wx.FONTFAMILY_DEFAULT,
                                 wx.FONTSTYLE_NORMAL,
                                 wx.FONTWEIGHT_BOLD))
        self.current = wx.StaticText(panel, -1, '\n')
        self.download_gauge = wx.Gauge(panel)
        self.status = wx.TextCtrl(panel, style=wx.TE_MULTILINE | wx.TE_READONLY)

        status = wx.BoxSizer()
        status.Add(self.cur, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        status.Add(self.current, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)

        main = wx.BoxSizer(wx.VERTICAL)
        main.Add(loading, 0, wx.ALIGN_CENTER_HORIZONTAL | wx.ALL, 5)
        main.Add(self.gauge, 0, wx.EXPAND | wx.ALL, 5)
        main.Add(status)
        main.Add(self.download_gauge, 0, wx.EXPAND | wx.ALL, 5)
        main.Add(self.status, 1, wx.EXPAND | wx.ALL, 5)

        panel.SetSizer(main)
        self.Center()
        self.Show()

        self.jobID = 0
        self.abortEvent = delayedresult.AbortEvent()
        self.Bind(wx.EVT_CLOSE, self.on_close)
        self.Bind(wx.EVT_TIMER, self.timer_handler)

        self.timer = wx.Timer(self)
        self.timer.Start(100)

        self.parent.Enable(False)
        self.handle_get(None)

    def timer_handler(self, event):
        self.gauge.Pulse()

    def on_close(self, event):
        """Only needed because in demo, closing the window does not kill the
        app, so worker thread continues and sends result to dead frame; normally
        your app would exit so this would not happen."""
        #if self.buttonAbort.IsEnabled():
            #self.log( "Exiting: Aborting job %s" % self.jobID )
            #self.handle_abort(None)
        self.Show()

    def handle_get(self, event):
        """Compute result in separate thread, doesn't affect GUI response."""
        self.abortEvent.clear()
        self.jobID += 1

        log.info("Starting job %s in producer thread: GUI remains responsive" %
                 self.jobID)
        delayedresult.startWorker(self._result_consumer,
                                  self._result_producer,
                                  wargs=(self.jobID, self.abortEvent),
                                  jobID=self.jobID)

    def _result_producer(self, jobID, abort_event):
        """Downloads the files in self.files"""
        self.numfiles = len(self.files)
        msg = _("Downloading ") + str(self.numfiles) + " " + _("file(s)") + "\n"
        wx.CallAfter(self.log_message, msg)

        downloader = Downloader()

        failed = self.files[:]
        self.numfile = 0
        for data in self.files:
            if abort_event():
                return [1, failed]
            url = data[0]
            filepath = data[1]
            end = url.split('/')[-1]
            if len(data) <= 2:
                checksum = {}
            else:
                checksum = data[2]

            msg = _('Starting') + ' ' + end
            wx.CallAfter(self.set_file, msg)
            # TODO: Change this to file progress (move to self.progress)

            self.curfile = end
            self.numfile += 1
            if os.path.exists(filepath):
                if not self.overwrite and self._verify(filepath, checksum) >= 1:
                    msg = _("Skipped: ") + end + "\n" + _("Reason: ") + \
                          _("File already exists.") + "\n"
                    wx.CallAfter(self.log_message, msg)
                    failed.remove(data)
                    continue
                else:
                    msg = _("Deleted: ") + filepath + "\n" + \
                          _("Reason: ") + \
                          _("Existing file failed checksum verify.") + "\n"
                    wx.CallAfter(self.log_message, msg)
                os.remove(filepath)
            retries = 0
            success = False
            while retries <= self.retries:
                try:  # Attempt to download the file
                    msg = _("Downloading: ") + url + "\n"
                    if retries != 0:
                        msg = _("Retrying: ") + "[%i/%i] " % (retries, self.retries) + url + "\n"
                    wx.CallAfter(self.log_message, msg)

                    downloader.retrieve(url, filepath, self.progress)
                    
                    if self._verify(filepath, checksum) == 0:
                        msg = _("Failed verify: ") + filepath + "\n"
                        wx.CallAfter(self.log_message, msg)
                        os.remove(filepath)
                        retries += 1
                        continue

                    success = True
                    failed.remove(data)
                    msg = _("Success: ") + filepath + "\n"
                    wx.CallAfter(self.log_message, msg)

                    if self.extract:
                        msg = _("Extracting") + " " + end
                        wx.CallAfter(self.set_file, msg)
                        try:
                            import gzip
                            infile = gzip.open(filepath, 'rb')
                            outfile = open(filepath.strip('.gz'), 'wb')
                            outfile.write(infile.read())
                            outfile.close()
                            infile.close()
                            os.remove(filepath)
                        except:  # Failed to extract
                            msg = _("Unable to extract: ") + filepath
                            wx.CallAfter(self.log_message, msg)
                    break

                except IOError as e:  # Failed downloading
                    msg = _("Failed: ") + url + "\n" + \
                          _("Reason: ") + str(e) + "\n"
                    wx.CallAfter(self.log_message, msg)
                    retries += 1

        msg = _("Downloaded: ") + str(len(self.files) - len(failed)) + ", " + \
              _("Failed: ") + str(len(failed)) + "\n"
        wx.CallAfter(self.log_message, msg)
        """if not failed:
            wx.CallAfter(wx.MessageBox,
                         _('All downloads have been completed successfully.'),
                         _('Download Complete'))
            result = [0, []]
        else:
            wx.CallAfter(wx.MessageBox,
                         _("Some downloads failed to complete.") +
                         "\n" + _("Please check") + " "
                         + config.log_path + " "
                         + _("for more details."), _("Download Failed"))
            result = [1, failed]"""
        return [0, []]#result

    def _verify(self,
                filename,
                checksum,
                cspriority=('SHA256', 'SHA1', 'MD5sum')):
        """Does a hash check on 'filename' as per hashes in 'checksum' dict. 
        Currently only supports sha256, sha1, md5.
        """
        if cspriority:
            checks = [x for x in cspriority if x in checksum]
            if not checks:
                return 2
        else:
            checks = checksum.keys()
        if checks:
            return 2

        if checks[0] == 'SHA256':
            check = hashlib.sha256()
        elif checks[0] == 'SHA1':
            check = hashlib.sha1()
        elif checks[0] == 'MD5sum':
            check = hashlib.md5()
        else:
            return 2
        
        with open(filename, 'rb') as fd:
            data = fd.read(1024*64)
            while data:
                check.update(data)
                data = fd.read(1024*64)
        if check.hexdigest() == checksum[checks[0]]:
            return 1
        return 0

    def progress(self, blocks, size, total):
        if blocks*size > total:
            fraction = float(total)/float(total)
        else:
            fraction = float(blocks*size)/float(total)

        wx.CallAfter(self.set_gauge, int(round(fraction * 100, 2)))
        msg = self.curfile + "\n" + str(int(round(fraction*100, 2))) + \
              " % of " + lib.convert_file_size(total) + " - File " + \
              str(self.numfile) + "/" + str(self.numfiles)
        wx.CallAfter(self.set_file, msg)

    def log_message(self, msg):
        log.info(msg[:-1])
        self.status.AppendText(msg)

    def set_gauge(self, val):
        self.download_gauge.SetValue(val)

    def set_file(self, val):
        self.current.SetLabel(val)

    def handle_abort(self, event):
        """Abort the result computation."""
        log.info("Aborting result for job %s" % self.jobID)
        #self.buttonGet.Enable(True)
        #self.buttonAbort.Enable(False)
        self.abortEvent.set()

    def _result_consumer(self, delayedResult):
        jobID = delayedResult.getJobID()
        assert jobID == self.jobID
        result = delayedResult.get()

        # output result
        #log.info( "Got result for job %s: %s" % (jobID, result) )
        self.parent.Enable()
        self.DestroyLater()
        if self.function:
            self.function()


class Downloader(urllib.request.FancyURLopener):
    def __init__(self):
        urllib.request.FancyURLopener.__init__(self, {})
