# -*- coding: utf-8 -*-
#
# Author: Chris Oliver (excid3@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import os.path
import sys
import webbrowser

import time
import wx
import wx.adv
import wx.lib.buttons as buttons
from wx.lib.mixins.listctrl import CheckListCtrlMixin

import lib
from lib import log, project
from .startDialog import StartDialog
from .misc import DetailsTab, ProportionalSplitter, VirtualList
from .delayedresult import Thread
from .download import Download
from .editor import FileEditor

from gettext import gettext as _

from lib import config


class MainApp(wx.Frame):
    def __init__(self, *args, **kwds):
        self.theme = config.theme
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)
        self.splitter = ProportionalSplitter(self)
        self.panel = wx.Panel(self.splitter)
        self.notebook = wx.Notebook(self.panel, -1, style=wx.NB_BOTTOM)
        self.project_notebook_pane = wx.Panel(self.notebook,
                                              -1,
                                              style=wx.TAB_TRAVERSAL)
        self.SetIcon(wx.Icon(config.ico_path, wx.BITMAP_TYPE_ICO))
        self.statusbar = self.CreateStatusBar(1, 0)

        # Side Buttons
        self.buttonPanel = wx.Panel(self)
        self.downloadButton = buttons.GenBitmapTextButton(self.buttonPanel,
                                                          -1,
                                                          wx.Bitmap(self.theme.icon_download),
                                                          _("Download"))
        self.updatesButton = buttons.GenBitmapTextButton(self.buttonPanel,
                                                         -1,
                                                         wx.Bitmap(self.theme.icon_updates),
                                                         _("Get Updates"))
        self.refreshButton = buttons.GenBitmapTextButton(self.buttonPanel,
                                                         -1,
                                                         wx.Bitmap(self.theme.icon_refresh),
                                                         _("Refresh"))

        self.notebook_il = wx.ImageList(16, 16)
        self.notebook_il.Add(wx.Bitmap(self.theme.icon_project_details))
        self.notebook.SetImageList(self.notebook_il)

        self.projectDetails = wx.TextCtrl(self.project_notebook_pane,
                                          style=wx.TE_MULTILINE | wx.TE_RICH | wx.TE_READONLY)
        self.details = DetailsTab(self.notebook)

        # Create list view
        self.list = VirtualList(self.splitter, self.details)

        # Create Popup menu for self.list
        self.__popup_menu()
        self.__create_menu()
        self.__set_properties()
        self.__do_layout()
        self.__bind_events()

    def __popup_menu(self):
        self.popupmenu = wx.Menu()
        # FIXME: popup menu items cannot display bitmap properly?
        item = self.popupmenu.Append(-1, _("Download"))
        self.Bind(wx.EVT_MENU, self.OnDownload, item)
        self.list.Bind(wx.EVT_CONTEXT_MENU, self.on_show_popup)

    def __create_menu(self):
        # Menu Bar
        self.menubar = wx.MenuBar()
        self.fileFile = wx.Menu()
        self.fileClose = wx.MenuItem(self.fileFile,
                                     wx.NewId(),
                                     _("&Close Project...\tCtrl+X"),
                                     _("Closes the current project"),
                                     wx.ITEM_NORMAL)
        self.fileClose.SetBitmap(wx.Bitmap(self.theme.icon_close))
        self.fileFile.Append(self.fileClose)
        self.fileFile.AppendSeparator()
        self.fileQuit = wx.MenuItem(self.fileFile,
                                    wx.NewId(),
                                    _("&Quit\tCtrl+Q"),
                                    _("Quit this application"),
                                    wx.ITEM_NORMAL)
        self.fileQuit.SetBitmap(wx.Bitmap(self.theme.icon_quit))
        self.fileFile.Append(self.fileQuit)
        self.menubar.Append(self.fileFile, _("&File"))

        tmp_menu = wx.Menu()
        self.projRefresh = wx.MenuItem(tmp_menu,
                                       wx.NewId(),
                                       _("&Refresh\tCtrl+R"),
                                       _("Refresh package list"),
                                       wx.ITEM_NORMAL)
        self.projRefresh.SetBitmap(wx.Bitmap(self.theme.icon_refresh))
        tmp_menu.Append(self.projRefresh)
        tmp_menu.AppendSeparator()
        self.projUpdates = wx.MenuItem(tmp_menu,
                                       wx.NewId(),
                                       _("Get &Updates\tCtrl+U"),
                                       _("Grabs all packages that have updates"),
                                       wx.ITEM_NORMAL)
        self.projUpdates.SetBitmap(wx.Bitmap(self.theme.icon_updates))
        tmp_menu.Append(self.projUpdates)
        self.projDownload = wx.MenuItem(tmp_menu,
                                        wx.NewId(),
                                        _("&Download Package\tCtrl+D"),
                                        _("Downloads selected package"),
                                        wx.ITEM_NORMAL)
        self.projDownload.SetBitmap(wx.Bitmap(self.theme.icon_download))
        tmp_menu.Append(self.projDownload)
        self.projInstall = wx.MenuItem(tmp_menu,
                                       wx.NewId(),
                                       _("&Install Packages..."),
                                       _("Helps you install packages onto "
                                         "Project Machine"),
                                       wx.ITEM_NORMAL)
        self.projInstall.SetBitmap(wx.Bitmap(self.theme.icon_install))
        tmp_menu.Append(self.projInstall)
        self.projUpdateStatus = wx.MenuItem(tmp_menu,
                                            wx.NewId(),
                                            _("&Update Status"),
                                            _("Regrabs the list of installed "
                                              "packages"),
                                            wx.ITEM_NORMAL)
        self.projUpdateStatus.SetBitmap(wx.Bitmap(self.theme.icon_refresh))
        tmp_menu.Append(self.projUpdateStatus)
        tmp_menu.AppendSeparator()
        self.projSources = wx.MenuItem(tmp_menu,
                                       wx.NewId(),
                                       _("&Edit Sources...\tCtrl+E"),
                                       _("Edit project sources"),
                                       wx.ITEM_NORMAL)
        self.projSources.SetBitmap(wx.Bitmap(self.theme.icon_sources))
        tmp_menu.Append(self.projSources)

        self.menubar.Append(tmp_menu, _("&Project"))

        tmp_menu = wx.Menu()
        self.helpHomepage = wx.MenuItem(tmp_menu,
                                        wx.NewId(),
                                        _("&Homepage"),
                                        _("Visit the website"),
                                        wx.ITEM_NORMAL)
        self.helpHomepage.SetBitmap(wx.Bitmap(self.theme.icon_home))
        tmp_menu.Append(self.helpHomepage)
        self.helpTutorial = wx.MenuItem(tmp_menu,
                                        wx.NewId(),
                                        _("&Tutorial"),
                                        _("Read the tutorial"),
                                        wx.ITEM_NORMAL)
        self.helpTutorial.SetBitmap(wx.Bitmap(self.theme.icon_book_open))
        tmp_menu.Append(self.helpTutorial)
        self.helpOnline = wx.MenuItem(tmp_menu,
                                      wx.NewId(),
                                      _("&Get Help Online"),
                                      _("Visit the forums"),
                                      wx.ITEM_NORMAL)
        self.helpOnline.SetBitmap(wx.Bitmap(self.theme.icon_help))
        tmp_menu.Append(self.helpOnline)
        self.helpTranslate = wx.MenuItem(tmp_menu,
                                         wx.NewId(),
                                         _("&Translate This Application"),
                                         _("Help translate to other languages"),
                                         wx.ITEM_NORMAL)
        self.helpTranslate.SetBitmap(wx.Bitmap(self.theme.icon_translate))
        tmp_menu.Append(self.helpTranslate)
        self.helpReport = wx.MenuItem(tmp_menu,
                                      wx.NewId(),
                                      _("R&eport A Problem"),
                                      _("Report a bug"),
                                      wx.ITEM_NORMAL)
        self.helpReport.SetBitmap(wx.Bitmap(self.theme.icon_bug_report))
        tmp_menu.Append(self.helpReport)
        tmp_menu.AppendSeparator()
        self.helpDonate = wx.MenuItem(tmp_menu,
                                      wx.NewId(),
                                      _("&Donate"),
                                      _("Make a donation"),
                                      wx.ITEM_NORMAL)
        self.helpDonate.SetBitmap(wx.Bitmap(self.theme.icon_donate))
        tmp_menu.Append(self.helpDonate)
        tmp_menu.AppendSeparator()
        self.helpAbout = wx.MenuItem(tmp_menu,
                                     wx.NewId(),
                                     _("&About..."),
                                     _("About " + config.name),
                                     wx.ITEM_NORMAL)
        self.helpAbout.SetBitmap(wx.Bitmap(self.theme.icon_about))
        tmp_menu.Append(self.helpAbout)
        self.menubar.Append(tmp_menu,
                            _("&Help"))
        self.SetMenuBar(self.menubar)
        # Menu Bar end

    def __bind_events(self):
        self.Bind(wx.EVT_MENU, self.on_close, self.fileClose)
        self.Bind(wx.EVT_MENU, self.on_quit, self.fileQuit)
        self.Bind(wx.EVT_MENU, self.on_homepage, self.helpHomepage)
        self.Bind(wx.EVT_MENU, self.on_tutorial, self.helpTutorial)
        self.Bind(wx.EVT_MENU, self.on_help_online, self.helpOnline)
        self.Bind(wx.EVT_MENU, self.on_translate, self.helpTranslate)
        self.Bind(wx.EVT_MENU, self.on_report, self.helpReport)
        self.Bind(wx.EVT_MENU, self.on_donate, self.helpDonate)
        self.Bind(wx.EVT_MENU, self.on_about, self.helpAbout)

        self.Bind(wx.EVT_MENU, self.OnRefresh, self.projRefresh)
        self.Bind(wx.EVT_MENU, self.OnUpdates, self.projUpdates)
        self.Bind(wx.EVT_MENU, self.OnDownload, self.projDownload)
        self.Bind(wx.EVT_MENU, self.OnInstall, self.projInstall)
        self.Bind(wx.EVT_MENU, self.OnUpdateStatus, self.projUpdateStatus)
        self.Bind(wx.EVT_MENU, self.OnSources, self.projSources)

        self.Bind(wx.EVT_BUTTON, self.OnDownload, self.downloadButton)
        self.Bind(wx.EVT_BUTTON, self.OnRefresh, self.refreshButton)
        self.Bind(wx.EVT_BUTTON, self.OnUpdates, self.updatesButton)
        self.Bind(wx.EVT_CLOSE, self.closing)

    def __set_properties(self):
        self.SetSize((700, 500))
        self.statusbar.SetStatusWidths([-1])
        # statusbar fields
        statusbar_fields = [_("Ready")]
        for i in range(len(statusbar_fields)):
            self.statusbar.SetStatusText(statusbar_fields[i], i)

        attr = wx.TextAttr()
        attr.SetTabs([400])
        self.projectDetails.SetDefaultStyle(attr)

    def __do_layout(self):
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer_bottom = wx.BoxSizer(wx.HORIZONTAL)
        sizer_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_buttons.Add(self.downloadButton, 0, wx.ALL, 3)
        sizer_buttons.Add(self.updatesButton, 0, wx.ALL, 3)
        sizer_buttons.Add(self.refreshButton, 0, wx.ALL, 3)
        self.buttonPanel.SetSizer(sizer_buttons)

        sizer_project = wx.BoxSizer(wx.VERTICAL)
        sizer_project.Add(self.projectDetails, 1, wx.EXPAND, 0)
        self.project_notebook_pane.SetSizer(sizer_project)
        self.notebook.InsertPage(0,
                                 self.project_notebook_pane,
                                 _("Project Details"),
                                 False,
                                 0)
        sizer_bottom.Add(self.notebook, 1, wx.EXPAND, 0)
        self.panel.SetSizer(sizer_bottom)
        self.notebook.SetSelection(0)

        self.splitter.SplitHorizontally(self.list, self.panel)
        sizer.Add(self.buttonPanel, 0, wx.EXPAND, 0)
        sizer.Add(self.splitter, 1, wx.EXPAND, 0)
        self.SetSizer(sizer)
        self.Layout()
        self.Centre()

    def on_show_popup(self, evt):
        pos = evt.GetPosition()
        pos = self.ScreenToClient(pos)
        self.PopupMenu(self.popupmenu, pos)

    def closing(self, event):  # Cleanup cleanup, everybody do your share
        log.info(_("Shutting down"))
        self.Destroy()

    def on_close(self, event):
        self.list.DeleteAllItems()
        self.projectDetails.Clear()
        self.list.tabpage.set_package()
        start = StartDialog()
        success = start.ShowModal()
        start.Destroy()

        if success == wx.ID_CANCEL:
            self.Close()
        else:
            self.Refresh(lib.project.current.get_data())

    def on_quit(self, event):
        self.Close()

    def on_homepage(self, event):
        webbrowser.open(config.homepage_url)

    def on_tutorial(self, event):
        webbrowser.open(config.tutorial_url)

    def on_help_online(self, event):
        webbrowser.open(config.help_url)

    def on_translate(self, event):
        webbrowser.open(config.translate_url)

    def on_report(self, event):
        webbrowser.open(config.bug_url)

    def on_donate(self, event):
        webbrowser.open(config.donate_url)

    def on_about(self, event):
        info = wx.adv.AboutDialogInfo()

        info.SetIcon(wx.Icon(config.logo_path, wx.BITMAP_TYPE_PNG))
        info.SetName(config.name)
        info.SetVersion(config.version)
        info.SetDescription(config.description)
        info.SetCopyright(config.copyright)
        info.SetWebSite(config.homepage_url)
        info.SetLicence(config.license)
        info.AddDeveloper(config.authors)
        info.AddDocWriter(config.docwriters)
        info.AddArtist(config.artists)
        info.AddTranslator(config.translators)

        wx.adv.AboutBox(info)

    def OnSources(self, event):
        proj = project.current
        name = proj.get_sources() + _(' - Sources Editor')
        edit = FileEditor(name, proj.get_sources())
        if edit.ShowModal() == wx.ID_OK:
            edit.txt.SaveFile(proj.get_sources())
            if wx.MessageBox(_("Sources have changed. Would you like to reload "
                               "the package list?"),
                             _("Save Successful"),
                             wx.YES_NO) == 2: #wx.ID_YES:
                self.OnRefresh(None)
        edit.Destroy()

    def OnDownload(self, event):
        # Downloads selected package and dependencies
        selected = self.list.get_selected_item()
        if selected:
            files = project.current.get_dependencies(selected.name)

        # Generate filename list.
        list = ""
        for f in files:
            list += f[0] + "\n"

        # Confirm downloads
        dlg = ScrolledMessageDialog(None,
                                    _("Confirm Download"),
                                    _("Do you wish to download the following")
                                    + " " + str(len(files)) + " " + _("files?"),
                                    list)
        result = dlg.ShowModal() == wx.ID_OK
        dlg.Destroy()

        # User doesn't want to download them
        if not result:
            return

        # Make sure packages directory exists
        dir = os.path.join(project.current.dir, 'packages')
        if not os.path.exists(dir):
            os.mkdir(dir)

        # FIXME: Return to the plugin for post download cleanup
        Download(self, None, files)

        # FIXME: If any package fails to download, set package installed version
        # to blank list of failed files now stored in frame[1] as returned by
        # download.py

    def OnInstall(self, event):
        # Get the necessary information for package installation
        projdir = project.current.dir
        spacksdir = os.path.join(projdir, 'packages')
        spacks = [x for x in os.listdir(spacksdir)
                  if os.path.isfile(os.path.join(spacksdir, x)) and
                  (os.path.splitext(x)[1] == '.deb')]
        packnames = sorted([x[:x.find('_')] for x in spacks])

        # Ask what packages need to be installed
        items = dict([(num+1, pkg) for (num, pkg) in zip(range(len(packnames)), packnames)])
        dlg = SelectPackagesDialog(None, -1, _("Install Packages"), items)
        result = dlg.ShowModal()
        packnames = dlg.GetSelected()
        dlg.Destroy()

        # Never mind about installing packages now...
        if result == wx.ID_CANCEL:
            return

        result = project.current.plugin.install_cache(project.current.dir, 'installcache')
        if not result:
            wx.MessageBox(_("Failed to update the APT cache. Aborting " \
                            "installation."), 
                            _("Failed to update cache"), wx.ICON_ERROR)
            return

        # Install the packages
        projdir = project.current.dir
        result = project.current.plugin.install_packs(projdir, ' '.join(packnames))
        if result:
            wx.MessageBox(_("Package installation finished. You should update "
                            "your project status (Project > Update Status)."), 
                          _("Installation finished"))
        else:
            wx.MessageBox(_("An error occurred while installing your "
                            "packages. Make sure no other package manager is "
                            "running when you try to Install the packages. "
                            "Check the log for more details.\n\nAnother "
                            "option is installing the packages by hand using "
                            "apt-get."), 
                          _("Install Error"))

    def OnUpdates(self, event):
        updates = []
        files = []
        for (key, value) in project.projects[len(project.projects) - 1].packages.iteritems():
            if value[0] == 1: updates.append(key)

        # Make sure we have updates
        if len(updates) == 0:
            return wx.MessageBox(_("No updates available. Your computer is "
                                   "up-to-date."),
                                 _("No updates"))

        # Got a list of package names, lets generate a list of filenames to
        # download
        for item in updates:
            files += project.current.get_dependencies(item)

        # Make sure packages directory exists
        dir = os.path.join(project.current.dir, 'packages')
        if not os.path.exists(dir):
            os.mkdir(dir)

        Download(self, None, files)

    def OnUpdateStatus(self, event):
        dlg = wx.MessageDialog(None,
                               _("This will update Keryx about which packages "
                                 "are installed on your computer. Only run "
                                 "this on the computer you created this "
                                 "project on.\n\nWould you like to continue?"),
                               _("Update Status"), wx.YES_NO | wx.ICON_QUESTION)
        result = dlg.ShowModal()
        dlg.Destroy()
        if result == wx.ID_NO:
            return
        success = self.UpdateStatus()
        if success:
            wx.MessageBox(_("Status update Succeeded."),
                          _("Status Update Succeeded"))
            self.loadLocal()
        else:
            wx.MessageBox(_("Status update failed. Try running Keryx as root "
                            "(using 'sudo')."),
                          _("Status Update Failed"),
                          wx.ICON_ERROR)

    def UpdateStatus(self):
        project_dir = project.current.dir
        return project.current.plugin.update_status(project_dir)

    def OnRefresh(self, event):
        self.Refresh(project.current.get_data())

    def Refresh(self, data):
        self.SetTitle(data[0] + " - " + config.name + " v" + config.version)
        self.projectDetails.Clear()
        self.projectDetails.AppendText(data[0] + '\n')
        if data[2] != '':
            self.projectDetails.AppendText(_("Operating System:") + '\t' + data[2] + '\n')
        if data[3] != '':
            self.projectDetails.AppendText(_("Version:") + '\t' + data[3] + '\n')
        if data[4] != '':
            self.projectDetails.AppendText(_("Kernel:") + '\t' + data[5] + '\n')
        if data[5] != '':
            self.projectDetails.AppendText(_("Architecture:") + '\t' + data[4] + '\n')
        self.projectDetails.AppendText(_("Drive Free Space:") + '\t' + lib.drive_free_space() + '\n')

        points = self.projectDetails.GetFont().GetPointSize()
        style = wx.Font(points, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD)
        self.projectDetails.SetStyle(0, len(data[0]), wx.TextAttr("black", wx.NullColour, style))

        # Get index file URLs.
        urls = project.current.get_urls()

        # Inquire whether latest package list should be downloaded
        urls_string = '\n'.join(['\n'.join(x) for x in urls])
        dlg = ScrolledMessageDialog(None,
                                    _("Download Latest?"),
                                    _("Download the latest package lists?"),
                                    urls_string)

        result = dlg.ShowModal()
        dlg.Destroy()

        # Load package list
        self.SetStatusText(_("Loading packages") + "...", 0)

        if result == wx.ID_OK:
            Download(self, self.loadLocal, urls, True)
        else:
            self.loadLocal()

    def loadLocal(self, result=None):
        #TODO: Verify downloads?
        Thread(self, project.current.load_local, self.loadPackages)

    def loadPackages(self, result=None):
        proj = project.current
        self.list.set_map(proj.packages)
        self.list.SortListItems(1)  # Sort column 1, ascending A->Z order

        self.SetStatusText(str(len(proj.packages.keys())) + " " + _("Packages"), 0)

# end of class MainApp


class ScrolledMessageDialog(wx.Dialog):
    def __init__(self, parent, title, caption, msg,
                 pos=wx.DefaultPosition, size=(400, 250),
                 style=wx.DEFAULT_DIALOG_STYLE):
        wx.Dialog.__init__(self, parent, -1, title, pos, size, style)
        x, y = pos
        if x == -1 and y == -1:
            self.CenterOnScreen(wx.BOTH)

        text1 = wx.StaticText(self, -1, caption, (5, 5), (370, 15))
        text2 = wx.TextCtrl(self, -1, msg, (5, 20), (370, 220),
                            style=wx.TE_MULTILINE | wx.TE_READONLY)

        sizer = wx.BoxSizer(wx.VERTICAL)

        btnSizer = wx.StdDialogButtonSizer()
        btn = wx.Button(self, wx.ID_CANCEL, 'No')
        btnSizer.SetCancelButton(btn)
        btnSizer.AddButton(btn)
        btn = wx.Button(self, wx.ID_OK, 'Yes')
        btnSizer.SetAffirmativeButton(btn)
        btn.SetDefault()
        btnSizer.AddButton(btn)
        btnSizer.Realize()

        sizer.Add(text1, 0, wx.EXPAND|wx.ALL, 5)
        sizer.Add(text2, 0, wx.EXPAND|wx.ALL, 5)
        sizer.Add(btnSizer, 0, wx.ALIGN_RIGHT | wx.TOP | wx.BOTTOM, 10)
        self.SetSizerAndFit(sizer)


class SelectPackagesDialog(wx.Dialog):
    """Dialog for selection of packages to be installed"""
    def __init__(self, parent, id, title, items):
        wx.Dialog.__init__(self, parent, id, title, size=(800, 600))

        vbox = wx.BoxSizer(wx.VERTICAL)

        btnSizer = wx.BoxSizer(wx.HORIZONTAL)
        btn = wx.Button(self, wx.ID_ANY, _("Select All"))
        self.Bind(wx.EVT_BUTTON, self.OnSelectAll, btn)
        btnSizer.Add(btn)
        btn = wx.Button(self, wx.ID_ANY, _("Select None"))
        self.Bind(wx.EVT_BUTTON, self.OnSelectNone, btn)
        btnSizer.Add(btn)

        vbox.Add(btnSizer, 0, wx.ALIGN_LEFT | wx.TOP | wx.BOTTOM)

        self.check_list = CheckListPanel(self, items)        
        vbox.Add(self.check_list, 1, wx.EXPAND)

        btnSizer = wx.StdDialogButtonSizer()
        btn = wx.Button(self, wx.ID_CANCEL, 'Cancel')
        btnSizer.SetCancelButton(btn)
        btnSizer.AddButton(btn)
        btn = wx.Button(self, wx.ID_OK, 'Continue')
        btn.SetDefault()
        btnSizer.SetAffirmativeButton(btn)
        btnSizer.AddButton(btn)
        btnSizer.Realize()

        vbox.Add(btnSizer, 0, wx.ALIGN_RIGHT | wx.TOP | wx.BOTTOM, 5)

        self.SetSizer(vbox)

    def GetSelected(self):
        selected = []
        for index in range(self.check_list.list.GetItemCount()):
            if self.check_list.list.IsChecked(index):
               selected.append(self.check_list.list.GetItemText(index))
        return selected

    def OnSelectAll(self, evt):
        for index in range(self.check_list.list.GetItemCount()):
            self.check_list.list.CheckItem(index, True)

    def OnSelectNone(self, evt):
        for index in range(self.check_list.list.GetItemCount()):
            self.check_list.list.CheckItem(index, False)


class CheckListCtrl(wx.ListCtrl, CheckListCtrlMixin):
    def __init__(self, parent):
        wx.ListCtrl.__init__(self, parent, -1, style=wx.LC_REPORT)
        CheckListCtrlMixin.__init__(self)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated)

    def OnItemActivated(self, evt):
        self.ToggleItem(evt.m_itemIndex)


class CheckListPanel(wx.Panel):
    def __init__(self, parent, items):
        wx.Panel.__init__(self, parent, -1)

        self.list = CheckListCtrl(self)
        sizer = wx.BoxSizer()
        sizer.Add(self.list, 1, wx.EXPAND)
        self.SetSizer(sizer)

        self.list.InsertColumn(0, _("Package"))

        for key, data in items.iteritems():
            index = self.list.InsertStringItem(sys.maxint, data)
            self.list.SetItemData(index, key)
            self.list.CheckItem(index, True)  # Check each item after it is added
      
        self.list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
