# -*- coding: utf-8 -*-
#
# Author: Chris Oliver (excid3@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import wx
import wx.lib.mixins.listctrl as listmix
from gettext import gettext as _

import lib
from lib import config
from lib.package import Package


class VirtualList(wx.ListCtrl,
                  listmix.ListCtrlAutoWidthMixin,
                  listmix.ColumnSorterMixin):
    def __init__(self, parent, tabpage, data={}):
        self.theme = config.theme
        wx.ListCtrl.__init__(self,
                             parent,
                             -1,
                             style=wx.LC_SINGLE_SEL |
                                   wx.LC_REPORT |
                                   wx.LC_VIRTUAL |
                                   wx.LC_HRULES |
                                   wx.LC_VRULES)
        self.tabpage = tabpage
        # Add some art
        self.il = wx.ImageList(16, 16)
        self.sm_up = self.il.Add(wx.Bitmap(self.theme.icon_arrow_up))
        self.sm_dn = self.il.Add(wx.Bitmap(self.theme.icon_arrow_down))
        self.uptodate = self.il.Add(wx.Bitmap(self.theme.icon_uptodate))
        self.update = self.il.Add(wx.Bitmap(self.theme.icon_update))
        self.error = self.il.Add(wx.Bitmap(self.theme.icon_error))
        self.downloaded = self.il.Add(wx.Bitmap(self.theme.icon_downloaded))
        self.SetImageList(self.il, wx.IMAGE_LIST_SMALL)

        # Add some attributes (colourful background for each item rows)
        self.attrUpdate = wx.ListItemAttr()
        self.attrUpdate.SetBackgroundColour("light blue")  # blue
        self.attrUptodate = wx.ListItemAttr()
        self.attrUptodate.SetBackgroundColour(wx.Colour(150, 235, 140))  # green
        self.attrError = wx.ListItemAttr()
        self.attrError.SetBackgroundColour(wx.Colour(255, 118, 106))  # red
        self.attrDownloaded = wx.ListItemAttr()
        self.attrDownloaded.SetBackgroundColour(wx.Colour(255, 215, 100))  # yellow

        # Build the columns
        count = 0

        columns = [(_('S'), 50),
                  (_('Package Name'), 200),
                  (_('Installed Version'), 200),
                  (_('Latest Version'), 200),
                  (_('Description'), 200)]

        for title, width in columns:
            self.InsertColumn(count, title)
            self.SetColumnWidth(count, width)
            count += 1

        # These two should probably be passed to init more cleanly setting the
        # numbers of items = number of elements in the dictionary
        self.set_map(data)
        self.currentItem = None

        # mixins
        listmix.ListCtrlAutoWidthMixin.__init__(self)
        listmix.ColumnSorterMixin.__init__(self, 3)

        # events
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_item_selected)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self._on_item_selected)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self._on_item_deselected)

    def set_map(self, data_map):
        self.itemDataMap = data_map
        self.itemIndexMap = data_map.keys()
        self.SetItemCount(len(data_map))
        self.tabpage.set_package()
        self.Refresh()

    def on_col_click(self, event):
        event.Skip()

    def get_selected_item(self):
        return self.itemDataMap[self.itemIndexMap[self.currentItem]]

    def on_item_selected(self, event):
        self.currentItem = event.Index
        index = self.itemIndexMap[self.currentItem]
        self.tabpage.set_package(self.itemDataMap[index])

    def _on_item_selected(self, event):
        self.currentItem = event.m_itemIndex

    def get_column_text(self, index, col):
        item = self.GetItem(index, col)
        return item.GetText()

    def _on_item_deselected(self, evt):
        pass

    def OnGetItemText(self, item, col):
        index = self.itemIndexMap[item]
        if col == 0:
            return self.itemDataMap[index].status
        elif col == 1:
            return self.itemDataMap[index].name
        elif col == 2:
            return self.itemDataMap[index].installed_version
        elif col == 3:
            return self.itemDataMap[index].version
        elif col == 4:
            return self.itemDataMap[index].description

    def OnGetItemImage(self, item):
        index = self.itemIndexMap[item]
        icon = self.itemDataMap[index].status

        if icon == 0:
            return self.uptodate
        elif icon == 1:
            return self.update
        elif icon == 2:
            return self.error
        elif icon == 3:
            return self.downloaded
        else:
            return -1

    def OnGetItemAttr(self, item):
        index = self.itemIndexMap[item]
        update = self.itemDataMap[index].status

        if update == 0:
            return self.attrUptodate
        elif update == 1:
            return self.attrUpdate
        elif update == 2:
            return self.attrError
        elif update == 3:
            return self.attrDownloaded
        else:
            return None

    def SortItems(self, sorter):
        items = list(self.itemDataMap.keys())
        items.sort()
        self.itemIndexMap = items

        # redraw the list
        self.Refresh()

    # Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py
    def GetListCtrl(self):
        return self

    # Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py
    def GetSortImages(self):
        return (self.sm_dn, self.sm_up)


class DetailsTab(object):
    def __init__(self, notebook):
        self.theme = config.theme
        self.notebook = notebook
        self.pane = wx.Panel(self.notebook, -1, style=wx.TAB_TRAVERSAL)
        index = self.notebook.GetImageList().\
            Add(wx.Bitmap(self.theme.icon_package))
        self.details = wx.TextCtrl(self.pane,
                                   style=wx.TE_MULTILINE |
                                         wx.TE_RICH |
                                         wx.TE_READONLY)

        attr = wx.TextAttr()
        attr.SetTabs([400])
        self.details.SetDefaultStyle(attr)

        sizer = wx.BoxSizer()
        sizer.Add(self.details, 1, wx.EXPAND, 0)
        self.pane.SetSizer(sizer)
        self.notebook.AddPage(self.pane, _("Package Details"), False, index)

    def set_package(self, stats=Package('')):
        self.details.Clear()
        if stats.name == '':
            return

        update = stats.status
        name = stats.name
        inst = stats.installed_version
        latest = stats.version
        descrip = stats.description
        depends = stats.depends
        filename = stats.filename
        size = lib.convert_file_size(stats.size)

        if not name == '':
            self.details.AppendText(name + '\n')
        if not descrip == '':
            self.details.AppendText(_("Description:\t") + descrip + '\n')
        if not inst == '':
            self.details.AppendText(_("Installed Version:\t") + inst + '\n')
        if not latest == '':
            self.details.AppendText(_("Latest Version:\t") + latest + '\n')
        if not depends == '':
            self.details.AppendText(_("Dependencies:\t") + depends + '\n')
        if not size == '':
            self.details.AppendText(_("Size:\t") + size + '\n')
        if not filename == '':
            self.details.AppendText(_("Filename:\t") + filename + '\n')

        # Fixes scrolling issue
        self.details.ScrollLines(-15)

        points = self.details.GetFont().GetPointSize()
        style = wx.Font(points,
                        wx.FONTFAMILY_DEFAULT,
                        wx.FONTSTYLE_NORMAL,
                        wx.FONTWEIGHT_BOLD)
        self.details.SetStyle(0,
                              len(name),
                              wx.TextAttr("black", wx.NullColour, style))


class ProportionalSplitter(wx.SplitterWindow):
    def __init__(self,
                 parent,
                 id=-1,
                 proportion=0.60,
                 size=wx.DefaultSize,
                 **kwargs):
            wx.SplitterWindow.__init__(self,
                                       parent,
                                       id,
                                       wx.Point(0, 0),
                                       size,
                                       **kwargs)
            self.SetMinimumPaneSize(50)
            self.proportion = proportion
            if not 0 < self.proportion < 1:
                raise ValueError("proportion value for "
                                 "ProportionalSplitter must be between 0 "
                                 "and 1.")
            self.ResetSash()
            self.Bind(wx.EVT_SIZE, self.OnReSize)
            self.Bind(wx.EVT_SPLITTER_SASH_POS_CHANGED,
                      self.OnSashChanged,
                      id=id)
            self.Bind(wx.EVT_PAINT, self.OnPaint)
            self.firstpaint = True

    def SplitHorizontally(self, win1, win2):
            if self.GetParent() is None: return False
            return wx.SplitterWindow.SplitHorizontally(self, win1, win2,
                    int(round(self.GetParent().GetSize().GetHeight() *
                              self.proportion)))

    def SplitVertically(self, win1, win2):
            if self.GetParent() is None: return False
            return wx.SplitterWindow.SplitVertically(self, win1, win2,
                    int(round(self.GetParent().GetSize().GetWidth() *
                              self.proportion)))

    def GetExpectedSashPosition(self):
        if self.GetSplitMode() == wx.SPLIT_HORIZONTAL:
            tot = max(self.GetMinimumPaneSize(),
                      self.GetParent().GetClientSize().height)
        else:
            tot = max(self.GetMinimumPaneSize(),
                      self.GetParent().GetClientSize().width)
        return int(round(tot * self.proportion))

    def ResetSash(self):
            self.SetSashPosition(self.GetExpectedSashPosition())

    def OnReSize(self, event):
            """Window has been resized, so we need to adjust the sash based on 
            self.proportion.
            """
            self.ResetSash()
            event.Skip()

    def OnSashChanged(self, event):
            """We'll change self.proportion now based on where user dragged the 
            sash.
            """
            pos = float(self.GetSashPosition())
            if self.GetSplitMode() == wx.SPLIT_HORIZONTAL:
                    tot = max(self.GetMinimumPaneSize(),
                              self.GetParent().GetClientSize().height)
            else:
                    tot = max(self.GetMinimumPaneSize(),
                              self.GetParent().GetClientSize().width)
            self.proportion = pos / tot
            event.Skip()

    def OnPaint(self, event):
            if self.firstpaint:
                    if self.GetSashPosition() != self.GetExpectedSashPosition():
                            self.ResetSash()
                    self.firstpaint = False
            event.Skip()
