# -*- coding: utf-8 -*-
#
# Author: Chris Oliver (excid3@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import os.path
import platform
import wx
import wx.lib.buttons as buttons

import lib.project
from lib.project import Project, ProjectExistsException
from lib import config
from lib import debian

from gettext import gettext as _


class StartDialog(wx.Dialog):
    """Startup dialog"""
    def __init__(self):
        self.theme = config.theme
        wx.Dialog.__init__(self, None, -1, _('Welcome to ') + config.name)
        self.SetIcon(wx.Icon(config.ico_path, wx.BITMAP_TYPE_ICO))
        self.bitmap = wx.StaticBitmap(self, -1, wx.Bitmap(config.logo_path))
        self.welcomeText = wx.StaticText(self,
                                         -1,
                                         _('Welcome to ') + config.name)
        self.welcomeText.SetFont(wx.Font(8,
                                         wx.FONTFAMILY_DEFAULT,
                                         wx.FONTSTYLE_NORMAL,
                                         wx.FONTWEIGHT_BOLD,
                                         0,
                                         ''))
        self.newText = wx.StaticText(self, -1, _('To get started, create a new '
                                                 'project on your offline '
                                                 'computer.'))
        self.nameTextCtrl = wx.TextCtrl(self, -1, platform.node())

        self.newProjectButton = \
            buttons.GenBitmapTextButton(self,
                                        -1,
                                        wx.Bitmap(self.theme.icon_new),
                                        _('New Project'))
        self.static_line = wx.StaticLine(self, -1)
        self.openTxt = wx.StaticText(self, -1, _('Open a project:'))
        self.openProjectBox = wx.Choice(self,
                                        -1,
                                        choices=[_('Browse...')],
                                        style=wx.CB_SORT)
        
        self.projects = []
        for thing in os.listdir(config.projects_dir):
            if os.path.isdir(os.path.join(config.projects_dir, thing)):
                self.openProjectBox.Append(thing)
                self.projects.append(os.path.join(config.projects_dir, thing))
                    
        self.openProjectBox.Select(1)

        self.openProjectButton = \
            buttons.GenBitmapTextButton(self,
                                        -1,
                                        wx.Bitmap(self.theme.icon_open),
                                        _('Open Project'))
        
        self.__do_layout()
        self.Bind(wx.EVT_BUTTON, self.on_new_project, self.newProjectButton)
        self.Bind(wx.EVT_BUTTON, self.on_open_project, self.openProjectButton)

    def __do_layout(self):
        # begin wxGlade: StartDialog.__do_layout
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1.Add(self.bitmap, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer_1.Add(self.welcomeText, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 3)
        sizer_1.Add(self.newText, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 3)
        sizer_2.Add(self.nameTextCtrl, 1, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 3)
        sizer_2.Add(self.newProjectButton, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 3)
        sizer_1.Add(sizer_2, 1, wx.EXPAND | wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 3)
        sizer_1.Add(self.static_line, 0, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_3.Add(self.openTxt, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 3)
        sizer_3.Add(self.openProjectBox, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 3)
        sizer_3.Add(self.openProjectButton, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_1.Add(sizer_3, 1, wx.ALIGN_CENTER_HORIZONTAL, 0)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        self.Layout()
        self.Centre()
        
    def on_new_project(self, event):  # wxGlade: StartDialog.<event_handler>
        project_name = self.nameTextCtrl.GetLineText(0)

        if project_name == '':
            wx.MessageBox(_('Please enter a project name.'), _('Error'))
            return

        # Set current project to a new project
        lib.project.current = Project()

        if not debian.is_os():
            wx.MessageBox(_('Please create the project on your offline '
                            'computer.'),
                          _('Error'))
            return

        # Create project
        try:
            lib.project.create_project(project_name)
        except ProjectExistsException:
            wx.MessageBox(_('Unable to create project. Make sure a project by '
                            'this name does not already exist.'), _('Error'))
            return

        lib.project.current.open_project(project_name)
        self.EndModal(wx.ID_OK)

    def on_open_project(self, event):  # wxGlade: startFrame.<event_handler>
        val = self.openProjectBox.GetItems()[self.openProjectBox.GetSelection()]
        if val != _('Browse...'): 
            open_path(os.path.join(config.projects_dir, val))
            self.EndModal(wx.ID_OK)
        else:
            dlg = wx.DirDialog(self,
                               message=_('Choose a folder'),
                               defaultPath=config.projects_dir,
                               style=wx.DD_DIR_MUST_EXIST)

            # Show the dialog and retrieve the user response. If it is the OK
            # response, process the data
            if dlg.ShowModal() == wx.ID_OK:
                paths = dlg.GetPath()
                dlg.Destroy()
                open_path(paths)
                self.EndModal(wx.ID_OK)


def open_path(path):
    # Create a new project
    lib.project.current = Project()

    # Load the project from file
    lib.project.current.open_project(path)
